from django.contrib.auth import get_user_model
from djangoldp.permissions import LDPPermissions
from rest_framework.reverse import reverse


class InboxPermissions(LDPPermissions):
    def has_permission(self, request, view):
        from djangoldp.models import Model

        if self.is_a_container(request._request.path):
            try:
                """
                If on nested field we use users permissions
                """
                obj = Model.resolve_parent(request.path)
                model = view.parent_model

                """
                If still on nested field and request is post (/users/X/inbox/) we use notification permissions
                """
                if view.parent_model != view.model and request.method == 'POST':
                    obj = None
                    model = view.model
            except:
                """
                Not on nested field we use notification permissions
                """
                obj = None
                model = view.model
        else:
            obj = Model.resolve_id(request._request.path)
            model = view.model

        perms = self.get_permissions(request.method, model)

        for perm in perms:
            if not perm.split('.')[1].split('_')[0] in self.user_permissions(request.user, model, obj):
                return False

        return True


class SubscriptionsPermissions(LDPPermissions):
    def has_permission(self, request, view):
        if request.user.is_anonymous and not request.method == "OPTIONS":
            return False

        if request.method in ["GET", "PATCH", "DELETE", "PUT"]:
            return True

        return super().has_permission(request, view)

    def has_object_permission(self, request, view, obj):
        if request.user.is_anonymous and not request.method == "OPTIONS":
            return False

        reverse_path_key = "{}-notification-list".format(get_user_model()._meta.object_name.lower())
        user_inbox = reverse(reverse_path_key, kwargs={"slug": request.user.slug}, request=request)
        if obj.inbox == user_inbox:
            return True

        return False



from django.db.models import signals


MODEL_MODIFICATION_USER_FIELD = 'modification_user'


class CurrentUserMiddleware:
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        self.process_request(request)
        response = self.get_response(request)
        signals.pre_save.disconnect(dispatch_uid=request)
        return response

    def process_request(self, request):
        if request.method in ('GET', 'HEAD', 'OPTION'):
            # this request shouldn't update anything
            # so no signal handler should be attached
            return

        if hasattr(request, 'user') and request.user.is_authenticated():
            user = request.user
        else:
            user = None

        def _update_users(sender, instance, **kwargs):
            setattr(instance, MODEL_MODIFICATION_USER_FIELD, user)

        signals.pre_save.connect(_update_users, dispatch_uid=request, weak=False)



import factory
from django.conf import settings
from django.apps import apps

from .models import Notification
from django.db.models.signals import post_save

@factory.django.mute_signals(post_save)
class NotificationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Notification

    type = factory.Faker('text', max_nb_chars=50)
    summary = factory.Faker('paragraph', nb_sentences=3, variable_nb_sentences=True)
    author = factory.Faker('url')
    user = factory.Iterator(apps.get_model(settings.AUTH_USER_MODEL).objects.all())
    date = factory.Faker('past_datetime')
    unread = factory.Faker('boolean')
    object = factory.Faker('url')


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-09-17 11:07
from __future__ import unicode_literals

from django.db import migrations
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='urlid',
            field=djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True),
        ),
    ]
